/* 
 * tkAppInit.c --
 *
 *	Provides a default version of the Tcl_AppInit procedure for
 *	use in wish and similar Tk-based applications.
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * Copyright (c) 1994 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) tkAppInit.c 1.21 96/03/26 16:47:07
 *
 * Modfications --
 *
 *   This code was last modified by Owen Lin on 7-8-97
 *   Commands needed for konnekt4 have been added. All supporting functions 
 *   start with the prefix K4Board_ . Refer to K4Board_Init for Tcl command
 *   names and corresponding C functions.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tk.h"
#include "K4Board.h"

extern int recursions;

/*
 * The following variable is a special hack that is needed in order for
 * Sun shared libraries to be used for Tcl.
 */

extern int matherr();
int *tclDummyMathPtr = (int *) matherr;

#ifdef TK_TEST
EXTERN int		Tktest_Init _ANSI_ARGS_((Tcl_Interp *interp));
#endif /* TK_TEST */

/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Results:
 *	None: Tk_Main never returns here, so this procedure never
 *	returns either.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

int
main(argc, argv)
    int argc;			/* Number of command-line arguments. */
    char **argv;		/* Values of command-line arguments. */
{
    Tk_Main(argc, argv, Tcl_AppInit);
    return 0;			/* Needed only to prevent compiler warning. */
}

/******************************************************************************
 * The following functions were added by Owen Lin
 *****************************************************************************/

/*-----------------------------------------------------------------------
 * inttostr -- This procedure converts a signed integer into a string.
 *             It will cause an error if the array passed in is smaller 
 *             than 7 elements
 *-----------------------------------------------------------------------
 */

void inttostr(int number, char str[]) {

  int temp;
  div_t div_result;
  int index;
  int i;
  int had_non_zero;

  index = 0;
  had_non_zero = 0;
  /* Handle the case if the number is equal to zero */
  if(number == 0) {
    str[index] = '0';
    index++;
    str[index] = NULL; 
    return;
  }
  /* Check if the number is negative */
  if(number < 0) {
    str[index] = '-';
    number = abs(number);
    index++;
  }
  /* Check what digit should be in the i's place */
  for(i = 10000; i > 0; i = i / 10) { 
    div_result = div(number,i);
    temp = div_result.quot;
    if(temp != 0) {      
      str[index] = temp + 48; /* add 48 to convert the number into it's ascii*/
      index++;                /* equivalent */
      number = number - temp * i; /* now subtract out the i's */
      had_non_zero = 1;
    }
    else if((temp == 0) && had_non_zero) {
      str[index] = '0';
      index++;
    }
  }
  /* place a NULL at the end of the string */  
  str[index] = NULL;  
}

/*
 *---------------------------------------------------------------------
 * K4Board_GetTurn -- This function implements the TCL command getturn.
 *                    
 * Results -- It returns a 1 if it is the red player's turn and a 0 if
 *            it the black player's turn.
 *
 *---------------------------------------------------------------------
 */

int K4Board_GetTurn(ClientData clientData, Tcl_Interp* interp, int argc, 
		    char* argv[]) {
  if(turn == 1) {
    interp->result = "1";
  }
  else {
    interp->result = "0";
  }
  return TCL_OK;  
}

/*
 *---------------------------------------------------------------------
 * K4Board_InitGame --
 *
 *---------------------------------------------------------------------
 */

int K4Board_InitGame(ClientData clientData, Tcl_Interp* interp, int argc, 
		    char* argv[]) {
  InitGame();
  return TCL_OK;
}

/*
 *---------------------------------------------------------------------
 * K4Board_ToggleTurn -- This function is the implementation of the
 *                       Tcl command toggleturn.
 *
 * Result -- The turn is toggle 0->1, 1->0
 *
 *---------------------------------------------------------------------
 */

int K4Board_ToggleTurn(ClientData clientData, Tcl_Interp* interp, int argc, 
		       char* argv[]) {
  ToggleTurn();
  return TCL_OK;  
}

/*
 *---------------------------------------------------------------------
 * K4Board_IsDraw -- This function is the implementation of the TCL
 *                   command isdraw.
 *
 * Result -- returns 1 if it is a draw and 0 otherwise.
 *
 *---------------------------------------------------------------------
 */

int K4Board_IsDraw(ClientData clientData, Tcl_Interp* interp, int argc, 
		    char* argv[]) {
  int result;

  result = IsDraw(&main_board);
  if(result == 1) {
    interp->result = "1";
  }
  else {
    interp->result = "0";
  }
  return TCL_OK;
}

/*
 *---------------------------------------------------------------------
 * K4Board_IsColumnFull -- This function is the C implementation of
 *                         the TCL command iscolumnfull. 
 *
 * Input -- takes as an argument the column number
 *
 * Results -- It returns 1 if it is full and 0 otherwise.
 *
 *---------------------------------------------------------------------
 */

int K4Board_IsColumnFull(ClientData clientData, Tcl_Interp* interp, int argc, 
			 char* argv[]) {
  int result;

  if((argc > 2) || (argc < 2)) {
    interp->result = "Wrong number of args!";
    return TCL_ERROR;
  }
  result = IsColumnFull(atoi(argv[1]),&main_board);
  if(result == 1) {
    interp->result = "1";
  }
  else {
    interp->result = "0";
  }
  return TCL_OK;  			
}
/*
 *---------------------------------------------------------------------
 * K4Board_ColumnHeight --
 *
 *---------------------------------------------------------------------
 */

int K4Board_ColumnHeight(ClientData clientData, Tcl_Interp* interp, int argc, 
			 char* argv[]) {
  int result;
  char number[7];
  
  if((argc > 2) || (argc < 2)) {
    interp->result = "Wrong number of args!";
    return TCL_ERROR;
  }
  result = ColumnHeight(atoi(argv[1]),&main_board);
  inttostr(result,number);  
  Tcl_AppendElement(interp,number);
  return TCL_OK;
}
/*
 *---------------------------------------------------------------------
 * K4Board_PutChip -- This function is the implementation of the Tcl
 *                    procedure putchip. 
 *
 *---------------------------------------------------------------------
 */

int K4Board_PutChip(ClientData clientData, Tcl_Interp* interp, int argc, 
		    char* argv[]) {
  if((argc > 3) || (argc < 3)) {
    interp->result = "Wrong number of args!";
    return TCL_ERROR;
  }
  PutChip(atoi(argv[1]),atoi(argv[2]),&main_board);
  return TCL_OK;
}

/*
 *---------------------------------------------------------------------
 * K4Board_RemoveChip --
 *
 *---------------------------------------------------------------------
 */

int K4Board_RemoveChip(ClientData clientData, Tcl_Interp* interp, int argc, 
		    char* argv[]) {
  if((argc > 2) || (argc < 2)) {
    interp->result = "Wrong number of args!";
    return TCL_ERROR;
  }
  RemoveChip(atoi(argv[1]),&main_board);
  return TCL_OK;
}

/*
 *---------------------------------------------------------------------
 * K4Board_Winner --
 *
 *---------------------------------------------------------------------
 */

int K4Board_Winner(ClientData clientData, Tcl_Interp* interp, int argc, 
		char* argv[]) {
  char number[7];
  int result[2];

  if((argc > 2) || (argc < 2)) {
    interp->result = "Wrong number of args!";
    return TCL_ERROR;
  }
  Winner(atoi(argv[1]),result,&main_board);
  inttostr(result[0],number);
  Tcl_AppendElement(interp,number);
  inttostr(result[1],number);
  Tcl_AppendElement(interp,number);
  return TCL_OK;
}

/*
 *---------------------------------------------------------------------
 * K4Board_GetPositionCoordinates --
 *
 *---------------------------------------------------------------------
 */

int K4Board_GetPositionCoordinates(ClientData clientData, Tcl_Interp* interp, 
				   int argc, char* argv[]) {
  int i;
  int position;
  char number[7];

  if((argc > 2) || (argc < 2)) {
    interp->result = "Wrong number of args!";
    return TCL_ERROR;
  }
    
  position = atoi(argv[1]);
  for(i = 0; i < 4; i++){
    inttostr(win_positions[position][i],number);
    Tcl_AppendElement(interp,number);
  }
  return TCL_OK;  
}


/*--------------------------------------------------------------------
 * K4Board_Dump -- This is the implementation of the Tcl commanf dump,
 *                 a debugging feature.
 *                 
 * Result -- Internal data structures are dumped to the screen. Those
 *           dumped are chips, chips_in_column, game_board, win_valid
 *--------------------------------------------------------------------
 */
int K4Board_Dump(ClientData clientData, Tcl_Interp* interp, int argc, 
			 char* argv[]) {
  int i;
  int j;

  printf("K4Board_Dump: main_board.chips = %i\n",main_board.chips);
  
  printf("K4Board_Dump: main_board.chips_in_column:\n");
  for(i = 1; i < 8; i++) {
    printf("%i ",main_board.chips_in_column[i]);
  }
  printf("\n");

  printf("K4Board_Dump: main_board.game_board:\n");
  for(i = 1; i < 8; i++) {
    for(j = 1; j < 7; j++) {
      printf("%i ",main_board.game_board[i][j]);
    }
    printf("\n");
  }

  printf("K4Board_Dump: main_board.win_valid:\n");
  for(i = 0; i < 2; i++) {
    for( j = 0; j < 69; j++) {
      printf("%i ",main_board.win_valid[i][j]);
    }
    printf("\n");
  }  
  return TCL_OK;
}

/*---------------------------------------------------------------------
 *
 *---------------------------------------------------------------------
 */
int K4Board_DFSMove(ClientData clientData, Tcl_Interp* interp, int argc, 
		    char* argv[]) {  
  int move;
  char number[7];
  
  if((argc > 3) || (argc < 3)) {
    interp->result = "Wrong number of args!"; 
    return TCL_ERROR; 
  } 
  
  recursions = 0;

  move = DFSMove2(atoi(argv[1]),atoi(argv[2]),&main_board);   
  inttostr(move,number);    
  Tcl_AppendElement(interp,number);   
  return TCL_OK;
}

/*---------------------------------------------------------------------
 *
 *---------------------------------------------------------------------
 */
int K4Board_OwensMove(ClientData clientData, Tcl_Interp* interp, int argc, 
		      char* argv[]) {
  int result[2];
  char number[7];
  
  if((argc > 3) || (argc < 3)) {
    interp->result = "Wrong number of args!"; 
    return TCL_ERROR; 
  } 
  
  recursions = 0;
  
  DFSMove(atoi(argv[1]),atoi(argv[2]),0,result,&main_board);  
  inttostr(result[0],number); 
  Tcl_AppendElement(interp,number);  
  return TCL_OK;
}

/*
 *---------------------------------------------------------------------
 * K4Board_Init -- This function creates all the commands implemented
 *                 in C for TCL. It also initializes the win_positins
 *                 array.
 *
 * Result -- The commands listed are valid Tcl commands and the array
 *           win_positions is valid.
 *
 *---------------------------------------------------------------------
 */

int K4Board_Init(Tcl_Interp* interp) {
 
  int i;
  int number1;
  int number2;
  int number3;
  int number4;
  FILE* infile;
  
  /* First, read in winning positions */
  infile = fopen("winpositions","r");
  for(i = 0; i < 69; i++) {
    fscanf(infile,"%i %i %i %i\n",&number1,&number2,&number3,&number4);   
    win_positions[i][0] = number1;
    win_positions[i][1] = number2;
    win_positions[i][2] = number3;
    win_positions[i][3] = number4;
  }
  fclose(infile);

  /* Now add the commands */
  Tcl_CreateCommand(interp,"getturn",K4Board_GetTurn, (ClientData) NULL,
		      (Tcl_CmdDeleteProc*) NULL);
  Tcl_CreateCommand(interp,"initgameaux",K4Board_InitGame, (ClientData) NULL,
		    (Tcl_CmdDeleteProc*) NULL);  
  Tcl_CreateCommand(interp,"toggleturnaux",K4Board_ToggleTurn, 
		    (ClientData) NULL, (Tcl_CmdDeleteProc*) NULL);
  Tcl_CreateCommand(interp,"isdraw",K4Board_IsDraw, (ClientData) NULL,
		    (Tcl_CmdDeleteProc*) NULL);  
  Tcl_CreateCommand(interp,"iscolumnfull",K4Board_IsColumnFull,
		    (ClientData) NULL,(Tcl_CmdDeleteProc*) NULL);  
  Tcl_CreateCommand(interp,"columnheight",K4Board_ColumnHeight, 
		    (ClientData) NULL, (Tcl_CmdDeleteProc*) NULL);  
  Tcl_CreateCommand(interp,"putchipaux",K4Board_PutChip,(ClientData) NULL,
		    (Tcl_CmdDeleteProc*) NULL);
  Tcl_CreateCommand(interp,"removechip",K4Board_RemoveChip,(ClientData) NULL,
		      (Tcl_CmdDeleteProc*) NULL);
  Tcl_CreateCommand(interp,"winner",K4Board_Winner,(ClientData) NULL,
		    (Tcl_CmdDeleteProc*) NULL); 
  Tcl_CreateCommand(interp,"getposcoords",K4Board_GetPositionCoordinates,
		    (ClientData) NULL, (Tcl_CmdDeleteProc*) NULL); 
  Tcl_CreateCommand(interp,"dump",K4Board_Dump, (ClientData) NULL,
		    (Tcl_CmdDeleteProc*) NULL);
  Tcl_CreateCommand(interp,"dfsmove",K4Board_DFSMove, (ClientData) NULL,
		    (Tcl_CmdDeleteProc*) NULL);
  Tcl_CreateCommand(interp,"owensmove",K4Board_OwensMove, (ClientData) NULL,
		    (Tcl_CmdDeleteProc*) NULL);
  return TCL_OK;
}

/******************************************************************************
 * End functions added by Owen Lin
 *****************************************************************************/

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization. *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int
Tcl_AppInit(interp)
    Tcl_Interp *interp;		/* Interpreter for application. */
{
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    Tcl_StaticPackage(interp, "Tk", Tk_Init, (Tcl_PackageInitProc *) NULL);
#ifdef TK_TEST
    if (Tktest_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    Tcl_StaticPackage(interp, "Tktest", Tktest_Init,
            (Tcl_PackageInitProc *) NULL);
#endif /* TK_TEST */


    /*
     * Call the init procedures for included packages.  Each call should
     * look like this:
     *
     * if (Mod_Init(interp) == TCL_ERROR) {
     *     return TCL_ERROR;
     * }
     *
     * where "Mod" is the name of the module.
     */
    if(K4Board_Init(interp) == TCL_ERROR) {
      return TCL_ERROR;
    }

    /*
     * Call Tcl_CreateCommand for application-specific commands, if
     * they weren't already created by the init procedures called above.
     */      
    
    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */

    Tcl_SetVar(interp, "tcl_rcFileName", "~/.konnekt4wishrc", TCL_GLOBAL_ONLY);
    return TCL_OK;
}
